/*
 * gc.c
 *
 * Copyright(c) 2004, END OF THE WORLD PRODUCTION CORPORATION,
 *                  & Alex S, a/k/a Particle, a/k/a Prof.Phreak
 *
 *                  alex@theparticle.com
 *                  http://www.theparticle.com
 */

/*

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 - March 4th, 2004 - Alex S.

*/

#include <windows.h>
#include <scrnsave.h>
#include <math.h>

#include "resource.h"

typedef BYTE byte_t;

#define DB_WIDTH 320
#define DB_HEIGHT 240

#define PI 3.14159265358979323846264338327950288419716939937510

struct pBITMAPINFO {
    BITMAPINFOHEADER bmiHeader;
    RGBQUAD bmiColors[256];
}BMInfo;

struct pLOGPALETTE {
    WORD    palVersion;
    WORD    palNumEntries;
    PALETTEENTRY palPalEntry[256];
}PalInfo;

HBITMAP hBM;
byte_t* double_buffer;
byte_t* double_double_buffer;

/**
 * plot a big `dot'
 */
void plot(int dotx,int doty){
    // plot our `moving' dot.

    if(dotx > DB_WIDTH-1)
        dotx = DB_WIDTH;
    if(dotx < 1)
        dotx = 1;
    if(doty > DB_HEIGHT-1)
        doty = DB_HEIGHT;
    if(doty < 1)
        doty = 1;

    double_double_buffer[(doty-1)*DB_WIDTH+(dotx-1)] = 255;
    double_double_buffer[(doty-1)*DB_WIDTH+(dotx)] = 255;
    double_double_buffer[(doty-1)*DB_WIDTH+(dotx+1)] = 255;
    double_double_buffer[doty*DB_WIDTH+(dotx-1)] = 255;
    double_double_buffer[doty*DB_WIDTH+dotx] = 255;
    double_double_buffer[doty*DB_WIDTH+(dotx+1)] = 255;
    double_double_buffer[(doty+1)*DB_WIDTH+(dotx-1)] = 255;
    double_double_buffer[(doty+1)*DB_WIDTH+dotx] = 255;
    double_double_buffer[(doty+1)*DB_WIDTH+(dotx+1)] = 255;
}

/**
 * function that gets called every few milliseconds
 */
void p_update(){

    static double angle1 = 0;
    static double angle2 = 0;
    static double angle3 = 0;

    int x,y,v,u,i,j;


    plot(cos(angle2)*sin(angle1)*160 + DB_WIDTH/2
         ,sin(angle2)*cos(angle1)*120 + DB_HEIGHT/2);
    plot(sin(angle2)*sin(angle1)*160 + DB_WIDTH/2
         ,cos(angle2)*cos(angle1)*120 + DB_HEIGHT/2);
    plot(cos(angle3)*sin(angle2)*160 + DB_WIDTH/2
         ,sin(angle3)*cos(angle2)*120 + DB_HEIGHT/2);
    plot(sin(angle3)*cos(angle1)*160 + DB_WIDTH/2
         ,cos(angle3)*sin(angle1)*120 + DB_HEIGHT/2);
    plot(sin(angle3)*cos(angle2)*160 + DB_WIDTH/2
         ,cos(angle2)*sin(angle1)*120 + DB_HEIGHT/2);
    plot(sin(angle2)*cos(angle1)*160 + DB_WIDTH/2
         ,cos(angle3)*sin(angle3)*120 + DB_HEIGHT/2);
    plot(cos(angle3)*sin(angle2)*sin(angle1)*160 + DB_WIDTH/2
         ,sin(angle3)*cos(angle2)*cos(angle1)*120 + DB_HEIGHT/2);

    for(i=0;i<4;i++){
        plot(rand()%DB_WIDTH,rand()%DB_HEIGHT);
    }

    for (y=1;y<DB_HEIGHT-1;y++) {
        for (x=1;x<DB_WIDTH-1;x++) {
            // do a `blur'
            int sum =
                +double_double_buffer[(y+-1)*DB_WIDTH+(x+-1)]
                +double_double_buffer[(y+-1)*DB_WIDTH+(x)]
                +double_double_buffer[(y+-1)*DB_WIDTH+(x+1)]
                +double_double_buffer[(y)*DB_WIDTH+(x+-1)]
                +double_double_buffer[(y)*DB_WIDTH+(x)] * 2
                +double_double_buffer[(y)*DB_WIDTH+(x+1)]
                +double_double_buffer[(y+1)*DB_WIDTH+(x+-1)]
                +double_double_buffer[(y+1)*DB_WIDTH+(x)]
                +double_double_buffer[(y+1)*DB_WIDTH+(x+1)];
            sum /= 10;
            double_buffer[y*DB_WIDTH+x] = (byte_t)sum;
        }
    }
    angle1 += PI / 320.0;
    if(angle1 > 2*PI)
        angle1 = 0;
    angle2 += PI / 180.0;
    if(angle2 > 2*PI)
        angle2 = 0;
    angle3 += PI / 120.0;
    if(angle3 > 2*PI)
        angle3 = 0;
    memcpy(double_double_buffer,double_buffer,sizeof(byte_t)*DB_WIDTH*DB_HEIGHT);
}

/**
 * function that figures out the palette (mostly yellowish).
 */
void getpalette(RGBQUAD* p){
    int i;

    p[0].rgbRed = (int)(0);
    p[0].rgbGreen = (int)(0);
    p[0].rgbBlue = (int)(0);
    p[0].rgbReserved = 0;

    for (i=1;i<256;i++) {
        p[i].rgbRed = i;
        p[i].rgbGreen = i;
        p[i].rgbBlue = i/4;
        p[i].rgbReserved = 0;
    }
}

/**
 * initialize double buffer
 */
void init_double_buffer(HDC hDC){
    HPALETTE PalHan;
    RGBQUAD palette[256];
    int i;

    double_double_buffer = (byte_t*)malloc(sizeof(byte_t)*DB_WIDTH*DB_HEIGHT);
    memset(double_double_buffer,0,DB_WIDTH*DB_HEIGHT);

    BMInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    BMInfo.bmiHeader.biWidth = DB_WIDTH;
    BMInfo.bmiHeader.biHeight = -abs(DB_HEIGHT);
    BMInfo.bmiHeader.biPlanes = 1;
    BMInfo.bmiHeader.biBitCount = 8;
    BMInfo.bmiHeader.biCompression = BI_RGB;
    BMInfo.bmiHeader.biSizeImage = 0;
    BMInfo.bmiHeader.biXPelsPerMeter = 0;
    BMInfo.bmiHeader.biYPelsPerMeter = 0;
    BMInfo.bmiHeader.biClrUsed = 256;
    BMInfo.bmiHeader.biClrImportant = 256;

    getpalette(palette);

    for (i=0;i<256;i++)
        BMInfo.bmiColors[i] = palette[i];

    PalInfo.palVersion = 0x300;
    PalInfo.palNumEntries = 256;
    for (i=0;i<256;i++) {
        PalInfo.palPalEntry[i].peRed = palette[i].rgbRed;
        PalInfo.palPalEntry[i].peGreen = palette[i].rgbGreen;
        PalInfo.palPalEntry[i].peBlue = palette[i].rgbBlue;
        PalInfo.palPalEntry[i].peFlags = PC_NOCOLLAPSE;
    }
    PalHan = CreatePalette((LOGPALETTE*)&PalInfo);
    SelectPalette(hDC,PalHan,FALSE);
    RealizePalette(hDC);
    DeleteObject(PalHan);
    hBM = CreateDIBSection(hDC,(BITMAPINFO*)&BMInfo,
                           DIB_RGB_COLORS,(void**)&double_buffer,0,0);
}

/**
 * copy double buffer to screen; stretching
 */
void blit(HDC hDC,RECT Dest){
    HDC Context;
    HBITMAP DefaultBitmap;
    Context = CreateCompatibleDC(0);
    DefaultBitmap = SelectObject(Context,hBM);
    StretchBlt(hDC,0,0,Dest.right,Dest.bottom,Context,0,0,DB_WIDTH,DB_HEIGHT,SRCCOPY);
    SelectObject(Context,DefaultBitmap);
    DeleteDC(Context);
    DeleteObject(DefaultBitmap);
}

/**
 * proc, initialize screen saver
 */
LRESULT WINAPI ScreenSaverProc(HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam){
    static HDC hdc,memdc;
    static RECT rect;
    static unsigned int timer;
    static HBITMAP hBitmap;
    static BYTE* double_buffer;

    switch (message) {
    case WM_CREATE:
        hdc = GetDC(hwnd);
        GetClientRect(hwnd,&rect);
        init_double_buffer(hdc);
        ReleaseDC(hwnd,hdc);
        timer = SetTimer(hwnd,1,33,NULL);
        break;
    case WM_TIMER:
        p_update();
        hdc = GetDC(hwnd);
        GetClientRect(hwnd,&rect);
        blit(hdc,rect);
        ReleaseDC(hwnd,hdc);
        break;
    case WM_ERASEBKGND:
        break;
    case WM_DESTROY:
        KillTimer(hwnd,timer);
        break;
    default:
        return DefScreenSaverProc(hwnd,message,wParam,lParam);
    }
    return 0;
}


BOOL WINAPI ScreenSaverConfigureDialog(HWND hdwnd,UINT message,WPARAM wParam,LPARAM lParam){
    return 0;

}

BOOL WINAPI RegisterDialogClasses(HINSTANCE hInst){
    return 1;
}

